package net.wanji.opt.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.bo.CrossIdBO;
import net.wanji.databus.vo.GreenwaveListVO;
import net.wanji.opt.bo.*;
import net.wanji.opt.dto.trend.AbnormalCrossListDTO;
import net.wanji.opt.dto.trend.EventAlarmDTO;
import net.wanji.opt.dto.trend.GreenwaveListDTO;
import net.wanji.opt.service.impl.TrendServiceImpl;
import net.wanji.opt.vo.*;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * 态势监测
 *
 * @author Kent HAN
 */
@Api(value = "TrendController", description = "态势监测")
@RequestMapping("/trend")
@RestController
public class TrendController {
    private final TrendServiceImpl trendService;

    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public TrendController(TrendServiceImpl trendService) {
        this.trendService = trendService;
    }

    @ApiOperation(value = "子区信息-路口监测", notes = "子区信息-路口监测", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveCrossMonitor",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenWaveCrossMonitorVO.class),
    })
    public JsonViewObject greenwaveCrossMonitor(@RequestBody GreenwaveIdBO greenwaveIdBO) {
        List<GreenWaveCrossMonitorVO> res = trendService.greenwaveCrossMonitor(greenwaveIdBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "子区信息-运行监测", notes = "子区信息-运行监测", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveRunMonitor",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenwaveRunMonitorVO.class),
    })
    public JsonViewObject greenwaveRunMonitor(@RequestBody GreenwaveIdBO greenwaveIdBO) {
        List<GreenwaveRunMonitorVO> res = trendService.greenwaveRunMonitor(greenwaveIdBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "干线方案详情", notes = "干线方案详情", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenwaveDetailVO.class),
    })
    public JsonViewObject greenwaveDetail(@RequestBody GreenwaveDetailBO bo) {
        GreenwaveDetailVO res = trendService.greenwaveDetail(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "干线当前详情", notes = "干线当前详情", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/currentGreenwaveDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenwaveDetailVO.class),
    })
    public JsonViewObject currentGreenwaveDetail(@RequestBody GreenwaveIdBO bo) throws Exception {
        GreenwaveDetailVO res = trendService.currentGreenwaveDetail(bo);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "保存子区策略", notes = "保存子区策略", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/saveGreenwaveStrategy",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = JsonViewObject.class),
    })
    public JsonViewObject saveGreenwaveStrategy(@RequestBody SaveGreenwaveStrategyBO saveGreenwaveStrategyBO) {
        trendService.saveGreenwaveStrategy(saveGreenwaveStrategyBO);
        return JsonViewObject.newInstance().success();
    }

    @ApiOperation(value = "子区路口指标", notes = "子区路口指标", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveCrossMetrics",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenwaveCrossMetricsVO.class),
    })
    public JsonViewObject greenwaveCrossMetrics(@RequestBody GreenwaveIdAndTimeStampBO greenwaveIdAndTimeStampBO) {
        List<GreenwaveCrossMetricsVO> res = trendService.greenwaveCrossMetrics(greenwaveIdAndTimeStampBO);
        return JsonViewObject.newInstance().success(res);
    }

    @ApiOperation(value = "问题子区列表", notes = "问题子区列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/greenwaveList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = GreenwaveVO.class),
    })
    public JsonViewObject greenwaveList(@RequestBody GreenwaveListDTO greenwaveListDTO) {
        GreenwaveVO greenwaveVO = new GreenwaveVO();
        List<GreenwaveListVO> greenwaveListVOList = trendService.greenwaveList(greenwaveListDTO);
        greenwaveVO.setGreenwaveList(greenwaveListVOList);
        // 绿波统计信息
        GreenwaveListDTO forStats = new GreenwaveListDTO();
        List<GreenwaveListVO> greenwaveListForStats = trendService.greenwaveList(forStats);
        GreenwaveStats greenwaveStats = trendService.buildGreenwaveStats(greenwaveListForStats);
        greenwaveVO.setGreenwaveStats(greenwaveStats);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(greenwaveVO);
    }

    @ApiOperation(value = "问题路口列表", notes = "问题路口列表", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/abnormalCrossList",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AbnormalCrossVO.class),
    })
    public JsonViewObject abnormalCrossList(@RequestBody AbnormalCrossListDTO abnormalCrossListDTO) {
        AbnormalCrossVO abnormalCrossVO = trendService.abnormalCrossList(abnormalCrossListDTO);
        return JsonViewObject.newInstance().success(abnormalCrossVO);
    }

    @ApiOperation(value = "问题路口详情", notes = "问题路口详情", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/abnormalCrossDetail",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = AbnormalCrossDetailVO.class),
    })
    public JsonViewObject abnormalCrossDetail(@RequestBody CrossIdBO crossIdBO) {
        AbnormalCrossDetailVO abnormalCrossDetailVO = trendService.abnormalCrossDetail(crossIdBO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(abnormalCrossDetailVO);
    }

    @ApiOperation(value = "事件告警", notes = "事件告警", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/eventAlarm",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject eventAlarm(@RequestBody EventAlarmDTO eventAlarmDTO) throws ParseException {
        String currentTimeStr = eventAlarmDTO.getCurrentTime();
        Date todayTime = sdf.parse(currentTimeStr);
        List<EventAlarmVO> eventAlarmVOList = trendService.eventAlarm(todayTime);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(eventAlarmVOList);
    }

    @ApiOperation(value = "数量实时推送", notes = "数量实时推送", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/countRealTime",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CountRealTimeVO.class),
    })
    public JsonViewObject countRealTime(@RequestBody CrossIdBO crossIdBO) throws Exception {
        String crossId = crossIdBO.getCrossId();
        CountRealTimeVO countRealTimeVO = trendService.countRealTime(crossId);
        return JsonViewObject.newInstance().success(countRealTimeVO);
    }

    @ApiOperation(value = "表格分时段查询", notes = "表格分时段查询", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/tableQuery",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = TableQueryVO.class),
    })
    public JsonViewObject tableQuery(@RequestBody CrossIdAndTimeSpanBO crossIdAndTimeSpanBO) throws Exception {
        TableQueryVO tableQueryVO = trendService.tableQuery(crossIdAndTimeSpanBO);
        return JsonViewObject.newInstance().success(tableQueryVO);
    }

    @ApiOperation(value = "表格实时推送", notes = "表格实时推送", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/tableRealTime",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = TableQueryVO.class),
    })
    public JsonViewObject tableRealTime(@RequestBody CrossIdAndIsFirstBO crossIdAndIsFirstBO) throws Exception {
        TableQueryVO tableRealTimeVO = trendService.tableRealTime(crossIdAndIsFirstBO);
        return JsonViewObject.newInstance().success(tableRealTimeVO);
    }

    @ApiOperation(value = "优化类型", notes = "优化类型", response = JsonViewObject.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/optType",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = OptTypeVO.class),
    })
    public JsonViewObject optType(@RequestBody CrossIdBO crossIdBO) {
        OptTypeVO optTypeVO = trendService.optType(crossIdBO);
        return JsonViewObject.newInstance().success(optTypeVO);
    }

    @ApiOperation(value = "重点路口监测", notes = "重点路口监测", response = JsonViewObject.class)
    @GetMapping(value = "/hotspotCross")
    public JsonViewObject hotspotCross() throws Exception {
        List<HotspotCrossVO> hotspotCrossVOS = trendService.hotspotCross();
        return JsonViewObject.newInstance().success(hotspotCrossVOS);
    }
}